package	jp.co.sds_corp.syslog.action;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.syslog.layout.ILayout;
import jp.co.sds_corp.syslog.layout.SyslogLayout;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
受信したsyslogを、ファイルに保存するアクションです。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR><TD>File</TD><TD>※</TD><TD>出力先</TD><TD>&nbsp;</TD></TR>
	<TR>
		<TD>Layout</TD><TD>&nbsp;</TD>
		<TD>formatに使用するLayout</TD>
		<TD>指定されない場合はメッセージのみ出力</TD>
	</TR>
	<TR>
		<TD>Filter</TD><TD>&nbsp;</TD>
		<TD>処理対象のsyslogを選択するFilter</TD>
		<TD>指定されない場合は、全てのsyslogが対象</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action class="jp.co.sds_corp.syslog.action.FileAction"&gt;
	&lt;File&gt;log/syslog.dat&lt;/File&gt;
	&lt;Layout class="jp.co.sds_corp.syslog.layout.SyslogLayout"&gt;
		%d %h %f:%p [%m]
	&lt;/Layout&gt;
&lt;/Action&gt;
</PRE>
*/
public class FileAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String	file;
	ILayout	fnLayout;

	public FileAction() {
		fnLayout = new SyslogLayout();
		fnLayout.setEscapeConversion( false );
	}

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setFile( config.get( "File" ));
		fnLayout.setFormat( getFile() );
	}

	/**
	* アクションを実行します。
	*/
	public void action( SyslogEvent ev ) {
		SyslogMsg msg = ev.getMessage();
		try {
			String s;
			if ( getLayout() == null ) {
				s = msg.getMessage();
			} else {
				s = getLayout().format( msg );
			}

			String file = fnLayout.format( msg );
			PrintWriter out = new PrintWriter(
				new BufferedWriter( new FileWriter( file, true )));
			out.print( s );
			out.flush();
			out.close();
		} catch ( Exception e ) {
			logger.error( "error", e );
		}
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = super.paramString();
		ret += file;
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		config.addChild( "File", getFile() );
	}

	/**
	* ファイル名(file)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFile( String var ) {
		this.file = var ;
		fnLayout.setFormat( var );
	}
	/**
	* ファイル名(file)を取得します。
	* @return	fileの内容
	*/
	public String getFile() {
		return file;
	}

}
