package	jp.co.sds_corp.syslog.action;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
Syslogを送信するアクションです。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR><TD>Host</TD><TD>※</TD><TD>出力先Host</TD><TD>&nbsp;</TD></TR>
	<TR><TD>Port</TD><TD>&nbsp;</TD>
		<TD>出力先Port</TD><TD>指定されない場合は514</TD>
	</TR>
	<TR><TD>Facility</TD><TD>&nbsp;</TD>
		<TD>出力するsyslogのFacility</TD>
		<TD>指定されない場合は受信したsyslogのfacility</TD>
	</TR>
	<TR><TD>Priority</TD><TD>&nbsp;</TD>
		<TD>出力するsyslogのPriority</TD>
		<TD>指定されない場合は受信したsyslogのpriority</TD>
	</TR>
	<TR><TD>Encode</TD><TD>&nbsp;</TD>
		<TD>出力するsyslogのメッセージのEncode</TD>
		<TD>指定されない場合はISO-8859-1</TD>
	</TR>
	<TR>
		<TD>Layout</TD><TD>&nbsp;</TD>
		<TD>formatに使用するLayout</TD>
		<TD>指定されない場合はメッセージのみ出力</TD>
	</TR>
	<TR>
		<TD>Filter</TD><TD>&nbsp;</TD>
		<TD>処理対象のsyslogを選択するFilter</TD>
		<TD>指定されない場合は、全てのsyslogが対象</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action class="jp.co.sds_corp.hibiya.action.SyslogAction"&gt;
	&lt;Host&gt;localhost&lt;/Host&gt;
	&lt;Port&gt;514&lt;/Port&gt;
	&lt;Facility&gt;LOCAL&lt;/Facility&gt;
	&lt;Priority&gt;ERROR&lt;/Priority&gt;
	&lt;Encode&gt;EUC-JP&lt;/Encode&gt;
	&lt;Layout class="jp.co.sds_corp.syslog.layout.SimpleLayout&gt;
		%hからメッセージを受信しました。
		%f:%p %m
	&lt;/Layout&gt;
&lt;/Action&gt;
</PRE>
*/
public class SyslogAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	String	host;
	int		port;
	String	facility;
	String	priority;
	String	encode;
	public static final String DEFAULT_ENCODE = "ISO-8859-1";

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setHost( config.get( "Host" ));
		setPort( config.getInteger( "Port", 514 ));
		setFacility( config.get( "Facility", null ));
		setPriority( config.get( "Priority", null ));
		setEncode( config.get( "Encode", DEFAULT_ENCODE ));
	}

	/**
	* アクションを実行します。
	*/
	public void action( SyslogEvent ev ) {
		SyslogMsg msg = ev.getMessage();
		String fac = getFacility();
		if ( fac == null ) {
			fac = msg.getFacility().toString();
		}
		String pri = getPriority();
		if ( pri == null ) {
			pri = msg.getPriority().toString();
		}
		String s;
		if ( getLayout() == null ) {
			s = msg.getMessage();
		} else {
			s = getLayout().format( msg );
		}

		try {
			InetAddress toAddr = InetAddress.getByName( getHost() );
			InetAddress localhost = InetAddress.getLocalHost();
			if ( getHost().equals( "localhost" ) ||
				( localhost != null && localhost.equals( toAddr )))
			{
				logger.warn( "can't forward message"
								+ " localhost=" + localhost
								+ " to=(" + getHost() + ") " + toAddr );
				return;
			}
			SyslogMsg sys = new SyslogMsg( fac, pri, s );
			byte[] buf = sys.getData().getBytes( getEncode() );
			DatagramPacket packet = new DatagramPacket( buf, buf.length );
			DatagramSocket sock = new DatagramSocket();
			sock.connect( toAddr, port );
			sock.send( packet );
		} catch ( Exception ex ) {
			logger.error( "error", ex );
		}
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = super.paramString();
		ret += host + ":" + port;
		if ( facility != null ) {
			ret += ":F=" + facility;
		}
		if ( priority != null ) {
			ret += ":P=" + priority;
		}
		ret += ":" + encode;
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		c = config.addChild( "Host", getHost() );
		c = config.addChild( "Port", String.valueOf( getPort()) );
		if ( getFacility() != null ) {
			c = config.addChild( "Facility", getFacility() );
		}
		if ( getPriority() != null ) {
			c = config.addChild( "Priority", getPriority() );
		}
		if ( getEncode() != null ) {
			c = config.addChild( "Encode", getEncode() );
		}
	}

	/**
	* 送信先ホスト(host)を設定します。
	* @param	var		設定するデータ
	*/
	public void setHost( String var ) {
		this.host = var ;
	}
	/**
	* 送信先ホスト(host)を取得します。
	* @return	hostの内容
	*/
	public String getHost() {
		return host;
	}

	/**
	* ポート番号(port)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPort( int var ) {
		this.port = var ;
	}
	/**
	* ポート番号(port)を取得します。
	* @return	portの内容
	*/
	public int getPort() {
		return port;
	}

	/**
	* Facility(facility)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFacility( String var ) {
		this.facility = var ;
	}
	/**
	* Facility(facility)を取得します。
	* @return	facilityの内容
	*/
	public String getFacility() {
		return facility;
	}

	/**
	* Priority(priority)を設定します。
	* @param	var		設定するデータ
	*/
	public void setPriority( String var ) {
		this.priority = var ;
	}
	/**
	* Priority(priority)を取得します。
	* @return	priorityの内容
	*/
	public String getPriority() {
		return priority;
	}

	/**
	* 文字セット名(encode)を設定します。
	* @param	var		設定するデータ
	*/
	public void setEncode( String var ) {
		if ( var == null || var.length() == 0 ) {
			var = DEFAULT_ENCODE;
		}
		this.encode = var ;
	}
	/**
	* 文字セット名(encode)を取得します。
	* @return	encodeの内容
	*/
	public String getEncode() {
		return encode;
	}

}
